import i18n from '@/i18n';
import config from '../../config.js';

import {
  Base,
  ForgottenPassword,
  NotFound,
  ReinitPassword,
  SignIn,
  SignOut,
  SignOutFailed,
  SignUp,
  SignUpSuccess,
  TermsOfUse,
  UserProfile,
  ValidationEmail,
  ValidationRegistration
} from '@/views';

import { createRouter, createWebHashHistory } from 'vue-router';

function normalizePathname(pathname) {
    const normalizedPath = decodeURI(pathname)
      // Replaces repeated slashes in the URL.
      .replace(/\/+/g, "/")
      // Reference: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/normalize
      // Note: Missing native IE support, may want to skip this step.
      .normalize()
  return `${normalizedPath}/`;
};

const routes = [
  {
    path: '/:locale',
    component: Base,
    beforeEnter: (to, from, next) => {
      const locale = to.params.locale;
      const supported_locales = import.meta.env.VITE_I18N_SUPPORTED_LOCALE.split(',');
      console.log(locale);

      if (!locale) {
        return next({
          path: `${import.meta.env.VITE_I18N_DEFAULT_LOCALE}${to.path.endsWith('/') ? to.path : normalizePathname(to.path)}`,
          query: to.query
        });
      }

      if (locale && locale.length !== 2) {
        return next({
          path: `${import.meta.env.VITE_I18N_DEFAULT_LOCALE}${to.path.endsWith('/') ? to.path : normalizePathname(to.path)}`,
          query: to.query
        });
      }

      if (locale && !supported_locales.includes(locale)) {
        return next({
          path:
            `${import.meta.env.VITE_I18N_DEFAULT_LOCALE}${to.path.slice(3).endsWith('/') ?
              to.path.slice(3) :
              normalizePathname(to.path.slice(3))}`,
          query: to.query
        });
      }

      if (!to.path.endsWith('/')) {
        return next({
          path: normalizePathname(to.path),
          query: to.query
        });
      }

      i18n.locale = locale;

      return next();

    },
    children: [
      {
        path: '',
        // redirect: { name: 'SignIn' },  ==> IT DOES NOT WORK!
        beforeEnter: (to, from, next) => {
          return next(`${to.path}signin/`);
        }
      },
      {
        path: 'signin',
        name: 'SignIn',
        component: SignIn,
      },
      {
        path: 'signup',
        name: 'SignUp',
        component: SignUp,
        beforeEnter: (to, from, next) => {
          if (config.forms.signup.disabled) {
            return next({ path: 'not-found/' });
          } else {
            return next();
          }
        }
      },
      {
        path: 'terms-of-use',
        name: 'TermsOfUse',
        component: TermsOfUse,
      },
      {
        path: 'signout',
        name: 'SignOut',
        component: SignOut,
      },
      {
        path: 'signout-failed',
        name: 'SignOutFailed',
        component: SignOutFailed,
      },
      {
        path: 'signupsuccess',
        name: 'SignUpSuccess',
        component: SignUpSuccess,
      },
      {
        path: 'validateregistration',
        name: 'ValidationRegistration',
        component: ValidationRegistration,
      },
      {
        path: 'validate-email',
        name: 'ValidationEmail',
        component: ValidationEmail,
      },
      {
        path: 'forgottenpwd',
        name: 'ForgottenPassword',
        component: ForgottenPassword,
      },
      {
        path: 'reinitpwd',
        name: 'ReinitPassword',
        component: ReinitPassword,
      },
      {
        path: 'profile',
        name: 'UserProfile',
        component: UserProfile,
      },
      {
        path: 'not-found',
        name: 'NotFound',
        component: NotFound,
      },
    ]
  },
  {
    path: '/:pathMatch(.*)*',
    name: 'Fallback',
    beforeEnter: (to, from, next) => {
      console.log(router.resolve(
        `${import.meta.env.VITE_I18N_DEFAULT_LOCALE}${to.path.endsWith('/') ?
          to.path :
          normalizePathname(to.path)}`
      ));
      const matchedRoutes = router.resolve(
        `${import.meta.env.VITE_I18N_DEFAULT_LOCALE}${to.path.endsWith('/') ?
          to.path :
          normalizePathname(to.path)}`
      ).resolved.matched;
      if (
        matchedRoutes && !matchedRoutes.find(el => el.name === 'Fallback')
      ) {
        return next({
          path: `${import.meta.env.VITE_I18N_DEFAULT_LOCALE}${to.path.endsWith('/') ? to.path : normalizePathname(to.path)}`,
          query: to.query
        });
      }
      return next({
        path: 'not-found/',
      });
    }
  }
];

const router = createRouter({
  history: createWebHashHistory(import.meta.env.BASE_URL),
  routes
});

export default router;
